﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Services;

using System.Data;
using GrapeCity.ActiveReports;

namespace mvc_unbound
{
    /// <summary>
    /// MyReportService の概要の説明です
    /// </summary>
    [WebService(Namespace = "http://tempuri.org/")]
    [WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
    [System.ComponentModel.ToolboxItem(false)]
    // この Web サービスを、スクリプトから ASP.NET AJAX を使用して呼び出せるようにするには、次の行のコメントを解除します。
    // [System.Web.Script.Services.ScriptService]
    public class MyReportService : GrapeCity.ActiveReports.Web.ReportService
    {
        // パラメータ
        private string reportParam;

        // レポート生成時に発生するイベント
        // Javascriptで指定した"reportID"が引数として渡される
        protected override object OnCreateReportHandler(string reportPath)
        {
            // パスにパラメータが含まれているかを判定
            string[] strSplit = reportPath.Split('?');

            // レポートパス
            reportPath = strSplit[0];

            if (strSplit.Length == 1)
            {
                // パラメータなし
                reportParam = "";

                // 通常処理
                return base.OnCreateReportHandler(reportPath);
            }

            // パラメータあり
            reportParam = strSplit[1];

            // 拡張子を取得
            string strExtension = System.IO.Path.GetExtension(reportPath);

            if (strExtension == ".rdlx")
            {
                // ページレポート/RDLレポートはここでは処理しない
                return base.OnCreateReportHandler(reportPath);
            }

            // セクションレポート
            SectionReport rpt = new SectionReport();

            if (strExtension == ".rpx")
            {
                // XML形式の場合、レイアウトファイル読込
                using (var xtr = new System.Xml.XmlTextReader(Server.MapPath(reportPath)))
                {
                    rpt.LoadLayout(xtr);
                }
            }
            else
            {
                // コード形式の場合、クラス名からインスタンスを生成
                Type t = Type.GetType(reportPath);
                rpt = (SectionReport)Activator.CreateInstance(t);
            }

            // データ設定
            rpt.DataSource = GetData();
            rpt.Run();

            return rpt;
        }

        // ページレポート/RDLレポートでアンバウンドデータソースを使用するときに発生するイベント
        // 詳細は製品ヘルプの以下のトピックをご参照ください。
        // 
        // ActiveReportsユーザーガイド
        //   - 基本操作
        //     - ページレポート／RDLレポートの基本操作
        //       - データ連結
        //         - 実行時にレポートとデータソースの連結
        //           - 「アンバウンドデータソース」
        //
        [Obsolete("例外的に使用します")]
        protected override void OnLocateDataSourceHandler(object sender, LocateDataSourceEventArgs args)
        {
            // ページレポート/RDLレポートはこのイベントでデータを設定する
            if (reportParam.ToString() != "")
            {
                args.Data = GetData();
            }
        }

        // データ取得用の関数
        private DataTable GetData() 
        {
            // パラメータにより、データパスを作成
            String dataPath = Server.MapPath("Data\\" + reportParam + ".mdb");

            // クエリ文字列
            String query = "Select * from Product";

            // データ接続
            System.Data.OleDb.OleDbConnection conn;
            String connString = "Provider=Microsoft.Jet.OLEDB.4.0;Data Source=" + dataPath;
            conn = new System.Data.OleDb.OleDbConnection(connString);
            var cmd = new System.Data.OleDb.OleDbCommand(query, conn);
            conn.Open();

            // データ取得
            var adapter = new System.Data.OleDb.OleDbDataAdapter();
            adapter.SelectCommand = cmd;
            var dtTable = new DataTable("Product");
            adapter.Fill(dtTable);

            return dtTable;     
        }

    }
}
