﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Services;

using System.Data;
using GrapeCity.ActiveReports;

namespace mvc_unbound
{
    /// <summary>
    /// Explains about a MyReportService
    /// </summary>
    [WebService(Namespace = "http://tempuri.org/")]
    [WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
    [System.ComponentModel.ToolboxItem(false)]
    // To make this web service callable by scripts using ASP.NET AJAX, delete the comment in the next line
    // [System.Web.Script.Services.ScriptService]
    public class MyReportService : GrapeCity.ActiveReports.Web.ReportService
    {
        // Parameter
        private string reportParam;

        // An event which occurs when the report is created
        // "reportID" which is specified with Javascript is passed as the argument
        protected override object OnCreateReportHandler(string reportPath)
        {
            // Determine whether a parameter is included in the path
            string[] strSplit = reportPath.Split('?');

            // Report path
            reportPath = strSplit[0];

            if (strSplit.Length == 1)
            {
                // Without parameter
                reportParam = "";

                // Processing normally
                return base.OnCreateReportHandler(reportPath);
            }

            // With parameter
            reportParam = strSplit[1];

            // Acquire the extension
            string strExtension = System.IO.Path.GetExtension(reportPath);

            if (strExtension == ".rdlx")
            {
                // In Page/RDL reports, this part is not processed.
                return base.OnCreateReportHandler(reportPath);
            }

            // In Section reports
            SectionReport rpt = new SectionReport();

            if (strExtension == ".rpx")
            {
                // When the report is in XML format, read the layout file
                using (var xtr = new System.Xml.XmlTextReader(Server.MapPath(reportPath)))
                {
                    rpt.LoadLayout(xtr);
                }
            }
            else
            {
                // When the report is code-based, create an instance from the class name
                Type t = Type.GetType(reportPath);
                rpt = (SectionReport)Activator.CreateInstance(t);
            }

            // Set the data
            rpt.DataSource = GetData();
            rpt.Run();

            return rpt;
        }

        // An event which occurs in Page/RDL reports when using unbound data sources
        // For more information, see the following topic in our User Guide
        // 
        // ActiveReports User Guide
        //   - How To
        //     - Page Report/RDL Report How To
        //       - Work with Data
        //         - Bind a Page Report to Data Source at Run Time
        //           - Unbound Data Source
        //
        [Obsolete("Used only for exceptional cases")]
        protected override void OnLocateDataSourceHandler(object sender, LocateDataSourceEventArgs args)
        {
            // The Page/RDL report sets data in this event
            if (reportParam.ToString() != "")
            {
                args.Data = GetData();
            }
        }

        // The function to retrieve data
        private DataTable GetData() 
        {
            // Create the data path by setting parameter
            String dataPath = Server.MapPath("Data\\" + reportParam + ".mdb");

            // Set a query string
            String query = "Select * from Product";

            // Data connection
            System.Data.OleDb.OleDbConnection conn;
            String connString = "Provider=Microsoft.Jet.OLEDB.4.0;Data Source=" + dataPath;
            conn = new System.Data.OleDb.OleDbConnection(connString);
            var cmd = new System.Data.OleDb.OleDbCommand(query, conn);
            conn.Open();

            // Data selection
            var adapter = new System.Data.OleDb.OleDbDataAdapter();
            adapter.SelectCommand = cmd;
            var dtTable = new DataTable("Product");
            adapter.Fill(dtTable);

            return dtTable;     
        }

    }
}
